import factory

from asaaspy.schemas.customer import CustomerCreateSchema


class CustomerCreateSchemaFactory(factory.Factory):
    class Meta:
        model = CustomerCreateSchema

    name = factory.Faker("name")
    cpfCnpj = factory.Faker("cpf", locale="pt_BR")



from asaaspy.client.base import AsaasClient
from asaaspy.client.resources.customer import CustomerResource
from asaaspy.client.resources.payment import PaymentResource


class AsaasService:
    def __init__(self, *args, **kwargs) -> None:
        """
        Inits a new AsaasPy Server instance.

        params:
            api_key(str): Asaas API Key to be used - required
            sandbox(bool): whether use or not sandbox (default False)
        """
        client = AsaasClient(*args, **kwargs)

        self.customer: CustomerResource = CustomerResource(client)
        self.payment: PaymentResource = PaymentResource(client)



import logging
import os

import pytest
import vcr

from asaaspy.service import AsaasService


@pytest.fixture
def asaas_svc(caplog, mock_http_client):
    caplog.set_level(logging.INFO)

    return AsaasService(
        api_key=os.environ.get("ASAAS_SANDBOX_TOKEN"),
        sandbox=True,
    )


@pytest.fixture
def mock_http_client():
    with vcr.use_cassette(
        "./tests/asaas_sandbox_vcr.yml",
        serializer="json",
        record_mode="new_episodes",
        filter_headers=["access_token"],
    ):
        yield



import httpx

from asaaspy.constants import APIBaseURL
from asaaspy.log import log
from asaaspy.schemas.base import PaginatedOutputPayload


class AsaasClient:
    def __init__(self, api_key, sandbox=False) -> None:
        self._client_params = {
            "headers": {
                "Content-Type": "application/json",
                "User-Agent": "AsaasPy",
                "access_token": api_key,
            },
            "base_url": (
                APIBaseURL.SANDBOX if sandbox else APIBaseURL.PRODUCTION
            ).value,
        }

        log.info(
            f"Initialized AsaasClient in {'sandbox' if sandbox else 'production'} mode"
        )

    def _get_http_client(self):
        log.info("Client instantiated")
        return httpx.Client(**self._client_params)


class AsaasResource:
    def __init__(self, asaas_client_instance):
        self._client = asaas_client_instance

    def get_client(self):
        return self._client._get_http_client()

    def get_list_response(
        self, response, data_response_class=dict
    ) -> PaginatedOutputPayload:
        # TODO: Work on a better way to handle HTTP RESPONSES
        response = PaginatedOutputPayload(**response.json())
        response.data = [data_response_class(**item) for item in response.data]

        return response



from datetime import date

from asaaspy.schemas.payment import (
    BillingType,
    PaymentCreateSchema,
    PaymentFilterBy,
    PaymentViewSchema,
)
from asaaspy.service import AsaasService


class TestAsaasCreatePayment:
    def test_create_minimal_payment(self, asaas_svc):
        asaas_svc: AsaasService = asaas_svc

        payment_create = PaymentCreateSchema(
            customer="cus_000006100224",
            billingType=BillingType.PIX,
            value=10.23,
            dueDate=date(year=2024, month=7, day=20),
        )

        payment_details = asaas_svc.payment.create(payment_create)

        assert payment_details.id.startswith("pay_")
        assert isinstance(payment_details, PaymentViewSchema)


class TestAsaasGetPayment:
    def test_get_payment(self, asaas_svc):
        asaas_svc: AsaasService = asaas_svc

        payment = asaas_svc.payment.get("pay_egd03gp8vsz433au")

        assert payment.id == "pay_egd03gp8vsz433au"
        assert payment.value == 10.23
        assert payment.dueDate == date(year=2024, month=7, day=20)


class TestAsaasAllPayment:
    def test_all_payment(self, asaas_svc):
        asaas_svc: AsaasService = asaas_svc

        all_payments = asaas_svc.payment.all()

        assert all_payments.data is not None
        assert isinstance(all_payments.data[0], PaymentViewSchema)

    def test_all_payment_filter_dto(self):
        filter_by = PaymentFilterBy(
            dateCreated_ge=date(2024, 7, 19), dueDate_le=date(2024, 8, 3)
        ).as_lean_dict()

        assert {
            "dateCreated[ge]": "2024-07-19",
            "dueDate[le]": "2024-08-03",
        } == filter_by

    def test_all_payment_with_filter(self, asaas_svc):
        asaas_svc: AsaasService = asaas_svc

        filtered_payments = asaas_svc.payment.all(PaymentFilterBy(status="RECEIVED"))

        assert filtered_payments.totalCount == 2


class TestAsaasDeletePayment:
    def test_delete_payment(self, asaas_svc):
        asaas_svc: AsaasService = asaas_svc

        has_deleted = asaas_svc.payment.delete(payment_id="pay_vtkxai01k8o8jo55")

        assert has_deleted



from asaaspy.client.base import AsaasResource
from asaaspy.schemas.base import PaginatedOutputPayload
from asaaspy.schemas.payment import (
    PaymentCreateSchema,
    PaymentFilterBy,
    PaymentViewSchema,
)


class PaymentResource(AsaasResource):
    def create(self, payment: PaymentCreateSchema) -> PaymentViewSchema:
        with self.get_client() as client:
            response = client.post("/v3/payments", json=payment.as_lean_dict())
            return PaymentViewSchema(**response.json())

    def get(self, payment_id) -> PaymentViewSchema:
        with self.get_client() as client:
            response = client.get(f"/v3/payments/{payment_id}")
            return PaymentViewSchema(**response.json())

    def all(self, filter_by: PaymentFilterBy = None) -> PaginatedOutputPayload:
        with self.get_client() as client:
            response = self.get_list_response(
                client.get(
                    "v3/payments",
                    params=filter_by.as_lean_dict() if filter_by else None,
                ),
                data_response_class=PaymentViewSchema,
            )
            return response

    def delete(self, payment_id) -> bool:
        with self.get_client() as client:
            response = client.delete(f"v3/payments/{payment_id}").json()
            return response.get("deleted", False)



from typing import Optional

from asaaspy.schemas.base import BaseSchema, QueryParamsPayload
from asaaspy.schemas.fields import Date


class CustomerCreateSchema(BaseSchema):
    name: str
    cpfCnpj: str
    email: Optional[str] = None
    phone: Optional[str] = None
    mobilePhone: Optional[str] = None
    address: Optional[str] = None
    addressNumber: Optional[str] = None
    complement: Optional[str] = None
    province: Optional[str] = None
    postalCode: Optional[str] = None
    externalReference: Optional[str] = None
    notificationDisabled: Optional[bool] = None
    additionalEmails: Optional[str] = None
    municipalInscription: Optional[str] = None
    stateInscription: Optional[str] = None
    observations: Optional[str] = None
    groupName: Optional[str] = None
    company: Optional[str] = None


class CustomerSchema(CustomerCreateSchema):
    id: Optional[str] = None
    object: Optional[str] = None
    personType: Optional[str] = None
    deleted: Optional[bool] = None
    canDelete: Optional[bool] = None
    canEdit: Optional[bool] = None
    cannotBeDeletedReason: Optional[bool] = None
    cannotEditReason: Optional[str] = None
    dateCreated: Optional[Date] = None

    city: Optional[str] = None
    cityName: Optional[str] = None

    state: Optional[str] = None
    country: Optional[str] = None


class CustomerSearchParams(QueryParamsPayload):
    name: Optional[str] = None
    email: Optional[str] = None
    cpfCnpj: Optional[str] = None
    groupName: Optional[str] = None
    externalReference: Optional[str] = None



from asaaspy.service import AsaasService, CustomerResource


class TestAsaasBaseClient:
    def test_customer_new(self, asaas_svc):
        asaas_svc: AsaasService = asaas_svc

        assert isinstance(asaas_svc.customer, CustomerResource)



import datetime

from pydantic import BeforeValidator, PlainSerializer
from typing_extensions import Annotated

Date = Annotated[
    datetime.date,
    BeforeValidator(
        lambda x: datetime.datetime.strptime(x, "%Y-%m-%d") if isinstance(x, str) else x
    ),
    PlainSerializer(lambda x: x.strftime("%Y-%m-%d")),
]



import pytest

from asaaspy.schemas.customer import CustomerCreateSchema, CustomerSchema
from asaaspy.service import AsaasService


class TestAsaasGetCustomer:
    def test_get_all_customers(self, asaas_svc):
        asaas_svc: AsaasService = asaas_svc
        all_customers = asaas_svc.customer.all()

        assert all_customers.totalCount == 3
        assert isinstance(all_customers.data[0], CustomerSchema)

    @pytest.mark.parametrize(
        "extra_params",
        ({"name": "Pedro"}, {"name": "Lucas", "email": "joaodascouves1@gmail.com"}),
    )
    def test_get_all_customers_with_filter(self, asaas_svc, extra_params):
        asaas_svc: AsaasService = asaas_svc

        all_customers = asaas_svc.customer.all()

        assert all_customers.hasMore is False


class TestAsaasCreateCustomer:
    def test_create_customer(self, asaas_svc):
        asaas_svc: AsaasService = asaas_svc

        customer = CustomerCreateSchema(name="Karen Baldwin", cpfCnpj="458.907.623-38")
        new_customer = asaas_svc.customer.create(customer)

        assert new_customer.id is not None


class TestAsaasDeleteCustomer:
    def test_delete(self, asaas_svc):
        asaas_svc: AsaasService = asaas_svc

        customer = CustomerCreateSchema(
            name="Usuario a Ser Excluido", cpfCnpj="083.138.660-63"
        )
        new_customer = asaas_svc.customer.create(customer)

        has_deleted = asaas_svc.customer.delete(id=new_customer.id)

        assert has_deleted



from asaaspy.client.base import AsaasResource
from asaaspy.schemas.customer import (
    CustomerCreateSchema,
    CustomerSchema,
    CustomerSearchParams,
)


class CustomerResource(AsaasResource):
    def all(
        self,
        name=None,
        email=None,
        cpf_cnpj=None,
        group_name=None,
        external_reference=None,
        **kwargs,
    ):
        with self.get_client() as client:
            response = self.get_list_response(
                client.get(
                    "v3/customers",
                    params=CustomerSearchParams(
                        **{
                            "name": name,
                            "email": email,
                            "cpfCnpj": cpf_cnpj,
                            "groupName": group_name,
                            "externalReference": external_reference,
                            **kwargs,
                        }
                    ).as_lean_dict(),
                ),
                data_response_class=CustomerSchema,
            )
            return response

    def create(self, customer: CustomerCreateSchema) -> CustomerSchema:
        with self.get_client() as client:
            response = client.post("v3/customers", json=customer.as_lean_dict())
            return CustomerSchema(**response.json())

    def delete(self, id: str) -> bool:
        with self.get_client() as client:
            response = client.delete(f"v3/customers/{id}").json()
            return response.get("deleted", False)
